;***********************************************************
;*     Electric keyboard emulator for PB-1000/C Ver.3      *
;*         (C)2006 Miyura (miyura321@ybb.ne.jp)            *
;***********************************************************

PB1000:	EQU	1		;PB-1000
PB1000E:EQU	2		;PB-1000 (overseas model)
PB1000C:EQU	3		;PB-1000C

;-----------------------------------------------------------
MODEL:	EQU	PB1000
;-----------------------------------------------------------

#if (MODEL=PB1000)|(MODEL=PB1000E)
EXPFN:	EQU	&HB21C
FNSCH:	EQU	&HE818
BRKJP:	EQU	&H9230
#else
EXPFN:	EQU	&HB342
FNSCH:	EQU	&HEA0C
BRKJP:	EQU	&H9300
#endif

DOTDS:	EQU	&H022E
LEDTP:	EQU	&H6201
WORK1:	EQU	&H68F4


	ORG	&H7000
	START	INIT
;
;	draw keyboard graphic data
;
INIT:	PRE	IZ,LEDTP
	XRM	$0,$0,8
	INVM	$0,8
	LD	$10,24
INT01:	STIM	$0,(IZ+$31),8	;line 1
	SB	$10,$30
	JR	NZ,INT01
	PRE	IX,CGDAT
	LDW	$2,ORDER
	LD	$10,96
INT02:	LD	$0,($2)
	LDM	$4,(IX+$0),6
	STIM	$4,(IZ+$31),6	;line 2-4
	ADW	$2,$30
	SB	$10,$30
	JR	NZ,INT02

	CAL	DOTDS
;
;	set sound volume
;
	LD	$29,&HC0
;
;	scan input key
;
KYSCN:	PRE	IX,KYDAT
	NA	$0,$31		;$0 = counter
KYS01:	LDIW	$11,(IX+$31)	;$11=IA, $12=KY
	PST	IA,$11
	AD	$0,$30
	SBC	$0,62
	JR	Z,KYSCN
	GRE	KY,$3
	SBC	$0,60
	JR	C,KYS02
	LD	$3,$4		;for [UP],[DOWN] keys
KYS02:	ANC	$3,$12		;check keycode
	JR	Z,KYS01		;

	SBC	$0,11		;check [BRK] key
	JP	Z,BRKJP
	SBC	$0,53		;check [ANS] key
	JR	Z,ATPLY
	SBC	$0,60
	JR	Z,INCVL		;check [UP] key
	JR	NC,DECVL	;check [DOWN] key
;
;	manual play mode
;
MNPLY:	CAL	DRAW		;draw finger
	CAL	BPSET		;
BEP01:	PST	PD,$2		;make a sound
	XR	$2,$29		;
	PRE	IX,$6
	SUP	$30		;wait
	GRE	KY,$0		;
	ANC	$3,$0		;key hold check
	JR	NZ,BEP01	;
	CAL	ERASE		;erase finger
	JR	KYSCN
;
;	volume control
;
INCVL:	AD	$29,&H80	;
	JR	NC,VOL01	;$29=00H->80H->C0H
	LD	$29,&HC0,VOL01	;
DECVL:	AN	$29,&H40	;
	JR	Z,VOL01		;$29=C0H->80H->00H
	LD	$29,&H80	;
VOL01:	GRE	KY,$0		;
	ANC	$12,$1		;key hold check
	JR	NZ,VOL01	;
	JR	KYSCN
;
;	auto play mode
;
ATPLY:	LDW	$15,FNAME	;
	LD	$17,7		;
	CAL	EXPFN		;set "MMC.BIN"
	LD	$4,&H0D		;
	CAL	FNSCH		;search "MMC.BIN"
	JR	C,KYSCN		;
	PRE	IX,$6		;
	LDM	$25,(IX+$30),4	;$25,$26 = initial address
	SBW	$27,$30		;$27,$28 = final address
;
;	initialize index address
;
CUE:	LDW	$11,$25		;initialize index address
;
;	scan input key
;
KYSCA:	PRE	IX,KYDT2
	NA	$6,$31		;$6 = counter
KYA01:	LDIW	$4,(IX+$31)	;$4=IA, $5=KY
	PST	IA,$4
	AD	$6,$30
	SBC	$6,7
	JR	Z,LOAD
	GRE	KY,$0
	SBC	$6,4
	JR	NC,KYA02
	LD	$0,$1		;for [LEFT],[RIGHT],[UP],[DOWN] keys
KYA02:	ANC	$5,$0		;check keycode
	JR	Z,KYA01

	SBC	$6,5		;check [0],[.] keys
	JR	Z,CUE
	JP	NC,KYSCN
	SBC	$6,3		;check [DOWN],[BRK] keys
	JR	Z,SCHUP
	JP	NC,BRKJP
	SBC	$6,$30		;check [LEFT],[UP] keys
	JR	Z,BT
	JR	NC,SCHDN
;
;	fast forward
;
FF:	CAL	SFTCK		;check [SHIFT] key
	JR	NZ,FF01
	LD	$0,20,FF02	;fast forward (index address + 20 )
FF01:	LD	$0,200		;fast forward (index address + 200)
FF02:	LD	$1,$31		;
	ADW	$11,$0		;
	SBCW	$11,$27		;check file end
	JP	NC,KYSCN
	JR	KYHLD
;
;	back turn
;
BT:	CAL	SFTCK		;check [SHIFT] key
	JR	NZ,BT01
	LD	$0,20,BT02	;back turn (index address - 20 )
BT01:	LD	$0,200		;back turn (index address - 200)
BT02:	LD	$1,$31		;
	SBW	$11,$0		;
	SBCW	$11,$25		;check file top
	JR	C,CUE		;cue
	JR	KYHLD
;
;	index search (backward)
;
SCHDN:	PRE	IX,$11		;set search area
	PRE	IY,$25		;file top to current
	SDN	&HC0
	JR	NZ,LOAD		;NZ->index pointer not found
	CAL	SFTCK		;check [SHIFT] key
	JR	Z,SCH01
	LDD	$0,(IX-2)	;IX=IX-2 (search area = file top to current - 2)
	SDN	&HC0
	JR	NZ,LOAD		;NZ->index pointer not found
	JR	SCH01
;
;	index search (forward)
;
SCHUP:	PRE	IX,$11		;set search area
	PRE	IY,$27		;current to file end
	SUP	&HC0
	JR	NZ,LOAD		;NZ->index pointer not found
SCH01:	LDD	$0,(IX+2)	;IX=IX+2
	GRE	IX,$11

KYHLD:	GRE	KY,$0		;
	ANC	$5,$1		;key hold check
	JR	NZ,KYHLD	;

LOAD:	LDW	$0,($11)	;$0=tone code, $1=length

	ADC	$0,$30		;check cue code
	JP	Z,CUE
	SBC	$1,$31		;null check
	JR	Z,INCID
	SBC	$1,65		;
	JR	C,RSTCK		;reset length
	LD	$1,64		;
RSTCK:	SBC	$0,60		;check rest code
	JR	NC,REST
;
;	calculate note length
;
	PRE	IX,LGDAT	;
	LD	$4,$0		;load length unit ($2,$3)
	BIU	$4		;
	LDW	$2,(IX+$4)	;
	SBCW	$2,$30		;null check
	JR	C,INCID		;
	XRW	$13,$13		;
LEN01:	ADW	$13,$2		;($13,$14) = ($2,$3) * $1
	SB	$1,$30		;
	JR	NZ,LEN01	;
	BIDW	$14		;
	BIDW	$14		;($13,$14) / 8
	BIDW	$14		;
;
;	draw finger
;
	CAL	DRAW
;
;	make a sound
;
	CAL	BPSET
BEP02:	PST	PD,$2
	XR	$2,$29
	PRE	IX,$6		;wait
	SUP	$30		;
	SBC	$0,$0		;adjust waiting time
	SBW	$13,$30
	JR	NZ,BEP02
;
;	erase finger
;
	CAL	ERASE
	JR	INCID
;
;	calculate rest length
;
REST:	LD	$13,$1
	LD	$14,$31
	DIUW	$13		;($13,$14) * 8
	BIDW	$14		;
;
;	wait for rest
;
	PRE	IY,&H300	;set bottom address of search area
RST01:	PRE	IX,&H110
	SUP	$30		;wait for rest
	SBW	$13,$30
	JR	NZ,RST01
;
;	increment index address
;
INCID:	ADW	$11,$30
	ADW	$11,$30
	SBCW	$11,$27		;check file end
	JP	NC,KYSCN
	JP	KYSCA
;
;	draw finger
;
DRAW:	LDW	$15,&HB07F	;
	LDW	$17,&HB0B0	;load finger graphic data
	LD	$19,&H7F	;
	PRE	IZ,BLACK	;set black key
	LD	$5,$0
	SBC	$0,29		;check G#2
	JR	Z,GS2DR		;
	JR	C,LCD1		;select LCD1/LCD2 ($8)
	SB	$5,30
	LDW	$8,&H0092	;LCD2
	JR	YSET
LCD1:	LDW	$8,&H0C82	;LCD1
;
;	set Y address
;
YSET:	LD	$6,$5
	BID	$6
	JR	C,YST01
	PRE	IZ,WHITE	;set white key
	LD	$10,3,XSET	;Y=$10 (line 4)
YST01:	LD	$10,2		;Y=$10 (line 3)
	AD	$9,6
;
;	set X address
;
XSET:	BIU	$6		;
	BIU	$6		;
	LD	$7,$6		;X = $9 + $6 * 12
	BIU	$7		;
	AD	$7,$6		;
	AD	$9,$7		;
	SBC	$9,96		;check LCD block
	JR	C,DRW01		;
	AD	$9,32		;add offset for block2
DRW01:	CAL	LCDST		;drawing
	STLM	$15,5		;
	RTN
;
;	erase finger
;
ERASE:	LDM	$15,(IZ+$31),5	;load key graphic data
	SBC	$5,29		;check G#2 key
	JR	NZ,DRW01	;
GS2DR:	LD	$10,2		;drawing for G#2 key
	LDW	$8,&HDA82	;LCD 1
	CAL	LCDST
	STLM	$15,3
	LDW	$8,&H0092	;LCD 2
	CAL	LCDST
	STLW	$18
	RTN
;
;	set LCD command
;
LCDST:	PPO	&HC5
	STLM	$8,3
	PPO	&HC4
	RTN
;
;	set beep
;
BPSET:	PRE	IX,TNDAT	;
	LD	$5,$0		;load tone data
	BIU	$0		;
	LDW	$6,(IX+$0)	;
	GST	PD,$2		;
	AN	$2,&H3F		;set PD resistor = 01xxxxxxB
	OR	$2,&H40		;
	PRE	IY,&H0300	;set bottom address of search area
	RTN
;
;	shift key check
;
SFTCK:	GST	IA,$2
	PST	IA,&HB
	NOP
	GRE	KY,$0
	ANC	$0,&H40
	PST	IA,$2
	RTN
;
;	key matrix data
;
KYDAT:	DB	&H4,&H20,&H4,&H10,&H5,&H20,&H4,&H08	; CAPS   A     Z     S
	DB	&H5,&H10,&H1,&H01,&H5,&H08,&H4,&H02	;  X           C     F
	DB	&H5,&H04,&H4,&H01,&H5,&H02,&H1,&H80	;  V     G     B    BRK
	DB	&H5,&H01,&H9,&H80,&H5,&H80,&H9,&H01	;  N     J     M     K
#if MODEL=PB1000E
	DB	&H7,&H01,&H9,&H02,&HB,&H40,&H1,&H01	; ENG    L   SHIFT
#else
	DB	&H7,&H01,&H9,&H02,&HA,&H01,&H1,&H01	; ENG    L    KANA
#endif
	DB	&H9,&H04,&H8,&H04,&H9,&H08,&H8,&H08	;  1     4     2     5
	DB	&H9,&H10,&H1,&H01,&H9,&H20,&H8,&H20	;  3           +     *
	DB	&HA,&H20,&H7,&H20,&H3,&H20,&HC,&H40	;  -     /     Q    FUNC
	DB	&H3,&H10,&H1,&H01,&H3,&H08,&H2,&H10	;  W           E     $
	DB	&H3,&H04,&H2,&H08,&H3,&H02,&H1,&H01	;  R     &     T
	DB	&H3,&H01,&H2,&H02,&H3,&H80,&H2,&H01	;  Y     ;     U     :
	DB	&H8,&H80,&H2,&H80,&H8,&H01,&H1,&H01	;  I     ,     O
	DB	&H8,&H02,&H6,&H80,&H7,&H02,&H6,&H01	;  P    CONT   7    CLS
	DB	&H7,&H04,&HA,&H08,&H7,&H08,&H6,&H02	;  8    ANS    9     BS
	DB	&H7,&H10,&H6,&H08,&H6,&H20,&H6,&H10	;  (    INS    )    STOP

	DB	&H5,&H80,&H4,&H80			;  UP   DOWN

KYDT2:	DB	&H3,&H80,&H5,&H40,&H5,&H80,&H4,&H80	;RIGHT  LEFT   UP   DOWN
	DB	&H1,&H80,&HA,&H02,&HA,&H04		; BRK    0     .
;
;	tone data
;
TNDAT:	DW	&H075,&H09A,&H0BD,&H0DE,&H0FD,&H000	; G0  G#0  A0  A#0  B0  --
	DW	&H11B,&H137,&H151,&H16A,&H182,&H000	; C1  C#1  D1  D#1  E1  --
	DW	&H198,&H1AD,&H1C0,&H1D3,&H1E4,&H1F5	; F1  F#1  G1  G#1  A1  A#1
	DW	&H205,&H000,&H213,&H221,&H22F,&H23B	; B1  --   C2  C#2  D2  D#2
	DW	&H247,&H000,&H252,&H25C,&H266,&H26F	; E2  --   F2  F#2  G2  G#2
	DW	&H278,&H280,&H288,&H000,&H290,&H297	; A2  A#2  B2  --   C3  C#3
	DW	&H29D,&H2A3,&H2A9,&H000,&H2AF,&H2B4	; D3  D#3  E3  --   F3  F#3
	DW	&H2B9,&H2BE,&H2C2,&H2C6,&H2CA,&H000	; G3  G#3  A3  A#3  B3  --
	DW	&H2CE,&H2D1,&H2D5,&H2D8,&H2DB,&H000	; C4  C#4  D4  D#4  E4  --
	DW	&H2DD,&H2E0,&H2E2,&H2E5,&H2E7,&H2E9	; F4  F#4  G4  G#4  A4  A#4
;
;	length data
;
LGDAT:	DW	049,052,055,058,062,000			; G0  G#0  A0  A#0  B0  --
	DW	065,069,073,078,083,000			; C1  C#1  D1  D#1  E1  --
	DW	087,093,098,104,110,117			; F1  F#1  G1  G#1  A1  A#1
	DW	124,000,131,139,147,156			; B1  --   C2  C#2  D2  D#2
	DW	165,000,175,185,196,207			; E2  --   F2  F#2  G2  G#2
	DW	220,233,247,000,263,278			; A2  A#2  B2  --   C3  C#3
	DW	293,310,329,000,350,370			; D3  D#3  E3  --   F3  F#3
	DW	393,418,440,466,494,000			; G3  G#3  A3  A#3  B3  --
	DW	526,553,593,627,666,000			; C4  C#4  D4  D#4  E4  --
	DW	694,742,777,837,883,933			; F4  F#4  G4  G#4  A4  A#4
;
;	keyboard graphic order data
;
ORDER:	DB	&H06,&H06,&H06,&H0C,&H00,&H06,&H0C,&H00 ;line 2
	DB	&H06,&H06,&H0C,&H00,&H06,&H0C,&H00,&H06
	DB	&H06,&H0C,&H00,&H06,&H0C,&H00,&H06,&H06
	DB	&H0C,&H00,&H06,&H0C,&H00,&H06,&H06,&H0C

	DB	&H24,&H18,&H18,&H1E,&H12,&H18,&H1E,&H12 ;line 3
	DB	&H18,&H18,&H1E,&H12,&H18,&H1E,&H12,&H18
	DB	&H18,&H1E,&H12,&H18,&H1E,&H12,&H18,&H18
	DB	&H1E,&H12,&H18,&H1E,&H12,&H18,&H18,&H1E

	DB	&H2A,&H30,&H30,&H30,&H30,&H30,&H30,&H30 ;line 4
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
;
;	graphic data
;
CGDAT:	DB	&H00,&H00,&H00,&H00,&HFF,&HFF		;1 (&H00)
	DB	&HFF,&H00,&H00,&H00,&HFF,&HFF		;2 (&H06)
	DB	&HFF,&H00,&H00,&H00,&H00,&HFF		;3 (&H0C)
	DB	&H00,&H00,&H00				;4 (&H12)
BLACK:	DB	&H00,&HFC,&HFF				;
	DB	&HFC,&H00,&H00,&H00,&HFC,&HFF		;5 (&H18)
	DB	&HFC,&H00,&H00,&H00,&H00,&HFF		;6 (&H1E)
	DB	&HFF,&H00,&H00,&H00,&HFC,&HFF		;7 (&H24)
	DB	&HFE,&H01,&H01,&H01,&H01,&HFE		;8 (&H2A)
WHITE:	DB	&H01,&H01,&H01,&H01,&H01,&HFE		;9 (&H30)
;
;	file name of music data
;
FNAME:	DB	"MMC.BIN"
