;************************************************************
;*  Electric keyboard emulator for FX-870P/VX-4/VX-3 Ver.3  *
;*       (C)2006 Miyura (miyura321@ybb.ne.jp) & Blue        *
;************************************************************

FX870P:	EQU	1		;FX-870P
VX3:	EQU	2		;VX-3

MODEL:	EQU	FX870P

#if MODEL=FX870P
SCHTP:	EQU	&H0109		;search area top
BASE:	EQU	&H1CD0		;start address
USER:	EQU	&H52FF		;CAL $15 (bank1)
BIOS:	EQU	&H5321		;CAL $28 (bank0)
DOTDS:	EQU	&H930F		;DOTDS routine
LEDTP:	EQU	&H123C		;LCD output buffer
DIREN:	EQU	&H18D1		;directory end pointer
FBFRES:	EQU	&H17A7		;FBF result code
#else
SCHTP:	EQU	&H0110		;search area top
BASE:	EQU	&H6CD0		;start address
USER:	EQU	&H5C85		;CAL $15 (bank1)
BIOS:	EQU	&H5CA7		;CAL $28 (bank0)
DOTDS:	EQU	&H1FD5		;DOTDS routine
LEDTP:	EQU	&H6343		;LCD output buffer
DIREN:	EQU	&H68CA		;directory end pointer
FBFRES:	EQU	&H67A0		;FBF result code
#endif

	ORG	BASE
	START	INIT
;
;	draw keyboard graphic
;
INIT:	PRE	IZ,LEDTP
	XRM	$0,$0,8
	INVM	$0,8
	LD	$10,24
INT01:	STIM	$0,(IZ+$31),8	;line 1
	SB	$10,$30
	JR	NZ,INT01
	PRE	IX,CGDAT
	LDW	$2,ORDER
#if MODEL=VX3
	LD	$10,96
#endif
INT02:	LD	$0,($2)
	LDM	$4,(IX+$0),6
	STIM	$4,(IZ+$31),6	;line 2-4
	ADW	$2,$30
#if MODEL=FX870P
	SBC	$0,&H3C
#else
	SB	$10,$30
#endif
	JR	NZ,INT02

	LDW	$28,DOTDS
	PST	UA,&H54
	CAL	BIOS
;
;	set sound volume
;
	LD	$29,&HC0

#if MODEL=FX870P
;
;	set indicators
;
	LD	$10,3		;
	CAL	SET01		;GRA indicator off
	STL	$31		;
IDMOD:	CAL	ANGST		;DEG indicator on
	STL	&H80		;
IDVOL:	CAL	CAPST		;CAPS indicator on/off
	STL	$29		;
	CAL	BASST		;BASIC indicator on/off
	STL	$29		;
#endif
;
;	scan input key
;
KYSCN:	PRE	IX,KYDAT
	NA	$0,$31		;$0 <- FFh
KYS01:	LDIW	$11,(IX+$31)	;$11=IA, $12=KY
	PST	IA,$11
	AD	$0,$30		;$0=$0+1
	SBC	$0,64
	JR	Z,KYSCN
	GRE	KY,$3
	SBC	$0,60
	JR	C,KYS02
	LD	$3,$4		;for [BS],[*],[+],[-] keys
KYS02:	ANC	$3,$12		;check keycode
	JR	Z,KYS01

	SBC	$0,11		;check [BRK] key
	JR	Z,QUIT
	SBC	$0,60		;check [BS] key
	JR	Z,QUIT
	SBC	$0,61		;check [*] key
	JR	Z,ATPLY
	SBC	$0,62
	JR	Z,INCVL		;check [+] key
	JR	NC,DECVL	;check [-] key
;
;	manual play mode
;
MNPLY:	CAL	DRAW		;draw finger
	CAL	BPSET		;
BEP01:	PST	PD,$2		;make a sound
	XR	$2,$29		;
	PRE	IX,$6
	SUP	$30		;wait
	GRE	KY,$0		;
	ANC	$3,$0		;key hold check
	JR	NZ,BEP01	;
	CAL	ERASE		;erase finger
	JR	KYSCN
;
;	volume control
;
INCVL:	AD	$29,&H80	;
	JR	NC,VOL01	;$29=00H->80H->C0H
	LD	$29,&HC0,VOL01	;
DECVL:	AN	$29,&H40	;
	JR	Z,VOL01		;$29=C0H->80H->00H
	LD	$29,&H80	;
VOL01:	GRE	KY,$0		;
	ANC	$12,$1		;key hold check
	JR	NZ,VOL01	;
#if MODEL=FX870P
	JR	IDVOL
#else
	JR	KYSCN
#endif
;
;	termination process
;
QUIT:	PST	UA,&H54
	RTN
;
;	auto play mode
;
ATPLY:	PRE	IX,DIREN	;
	LDW	$15,(IX+$31)	;get start address of FBF/VX-MENU
	ADW	$15,$30		;
	LD	$0,($15)	;
	SBC	$0,&H37		;confirm FBF/VX-MENU installation
	JR	NZ,KYSCN	;
;
;	search MMC file
;
	PHS	$29		;save volume data
	PRE	IZ,FNAME	;set file search command
	LDW	$28,USER	;
	PST	UA,&H54		;call FBF/VX-MENU
	CAL	BIOS		;
	PPS	$29		;load volume data
;
;	calculate MMC file address
;
	PRE	IX,FBFRES-8	;
	LDIM	$21,(IX+$31),8	;$25,$26 = initial address
	LD	$0,(IX+$31)	;$0 = result code
	ANC	$0,$30		;MMC exist->Z, none->NZ
	JP	NZ,KYSCN	;

	LDW	$27,$25		;
	ADW	$27,$21		;$27,$28 = final address
	SBW	$27,$30		;
;
;	set bank of MMC file
;
	LD	$20,&H55
	DID	$0
	BIU	$0
	AD	$20,$0

#if MODEL=FX870P
	CAL	ANGST		;RAD indicator on
	STL	$30
#endif
;
;	initialize index address
;
CUE:	LDW	$11,$25
;
;	scan input key
;
KYSCA:	PRE	IX,KYDT2
	NA	$6,$31
KYA01:	LDIW	$4,(IX+$31)	;$4=IA, $5=KY
	PST	IA,$4
	AD	$6,$30		;$6=$6+1
	SBC	$6,6
	GRE	KY,$0
	JR	C,KYA02
	JR	NZ,LOAD
	LD	$0,$1		;for [/] key
KYA02:	ANC	$5,$0		;check keycode
	JR	Z,KYA01

	SBC	$6,5		;check [CALC],[/] keys
	JR	Z,CUE
#if MODEL=FX870P
	JP	NC,IDMOD
#else
	JP	NC,KYSCN
#endif
	SBC	$6,3		;check [UP],[BRK] keys
	JR	Z,SCHUP
	JR	NC,QUIT
	SBC	$6,$30		;check [LEFT],[DOWN] keys
	JR	Z,BT
	JR	NC,SCHDN
;
;	fast forward
;
FF:	CAL	SFTCK		;check [SHIFT] key
	JR	NZ,FF01
	LD	$0,20,FF02	;fast forward (index address + 20 )
FF01:	LD	$0,200		;fast forward (index address + 200)
FF02:	LD	$1,$31		;
	ADW	$11,$0		;
	SBCW	$11,$27		;check file end
#if MODEL=FX870P
	JP	NC,IDMOD
#else
	JP	NC,KYSCN
#endif
	JR	KYHLD
;
;	back turn
;
BT:	CAL	SFTCK		;check [SHIFT] key
	JR	NZ,BT01
	LD	$0,20,BT02	;back turn (index address - 20 )
BT01:	LD	$0,200		;back turn (index address - 200)
BT02:	LD	$1,$31		;
	SBW	$11,$0		;
	SBCW	$11,$25		;check file top
	JR	C,CUE		;cue
	JR	KYHLD
;
;	index search (backward)
;
SCHDN:	PST	UA,$20
	PRE	IX,$11		;set search area
	PRE	IY,$25		;file top to current
	SDN	&HC0
	JR	NZ,LOAD		;NZ->index pointer not found
	CAL	SFTCK		;check [SHIFT] key
	JR	Z,SCH01
	LDD	$0,(IX-2)	;IX=IX-2
	SDN	&HC0
	JR	NZ,LOAD		;NZ->index pointer not found
	JR	SCH01
;
;	index search (forward)
;
SCHUP:	PST	UA,$20
	PRE	IX,$11		;set search area
	PRE	IY,$27		;current to file end
	SUP	&HC0
	JR	NZ,LOAD		;NZ->index pointer not found
SCH01:	LDD	$0,(IX+2)	;IX=IX+2
	GRE	IX,$11

KYHLD:	GRE	KY,$0		;
	ANC	$5,$0		;key hold check
	JR	NZ,KYHLD	;
;
;	load MMC data
;
LOAD:	PST	UA,$20
	LDW	$0,($11)	;$0=tone code, $1=length
	PST	UA,&H55

	ADC	$0,$30		;check cue code
	JP	Z,CUE
	SBC	$1,$31		;null check
	JR	Z,INCID
	SBC	$1,65		;
	JR	C,RSTCK		;reset length
	LD	$1,64		;
RSTCK:	SBC	$0,60		;check rest code
	JR	NC,REST
;
;	calculate note length
;
	PRE	IX,LGDAT	;
	LD	$4,$0		;load length unit ($2,$3)
	BIU	$4		;
	LDW	$2,(IX+$4)	;
	SBCW	$2,$30		;null check
	JR	C,INCID		;
	XRW	$13,$13		;
LEN01:	ADW	$13,$2		;($13,$14) = ($2,$3) * $1
	SB	$1,$30		;
	JR	NZ,LEN01	;
	BIDW	$14		;
	BIDW	$14		;($13,$14) / 8
	BIDW	$14		;
;
;	draw finger
;
	CAL	DRAW
;
;	make a sound
;
	CAL	BPSET
BEP02:	PST	PD,$2
	XR	$2,$29
	PRE	IX,$6		;wait
	SUP	$30		;
	SBC	$0,$0		;adjust waiting time
	SBW	$13,$30
	JR	NZ,BEP02
;
;	erase finger
;
	CAL	ERASE
	JR	INCID
;
;	calculate rest length
;
REST:	LD	$13,$1
	LD	$14,$31
	DIUW	$13		;($13,$14) * 8
	BIDW	$14		;
;
;	wait for rest
;
	CAL	BPS01
RST01:	PRE	IX,SCHTP
	SUP	$30
	SBW	$13,$30
	JR	NZ,RST01
	PST	UA,&H55		;IX = bank1
;
;	increment index address
;
INCID:	ADW	$11,$30
	SBCW	$11,$27		;check file end
#if MODEL=FX870P
	JP	Z,IDMOD
#else
	JP	Z,KYSCN
#endif
	ADW	$11,$30
	JP	KYSCA
;
;	draw finger
;
DRAW:	LDW	$15,&HB07F	;
	LDW	$17,&HB0B0	;load finger graphic data
	LD	$19,&H7F	;
	PRE	IZ,BLACK	;set black key
	LD	$5,$0
	SBC	$0,29		;check G#2
	JR	Z,GS2DR		;
	JR	C,LCD1		;select LCD1/LCD2 ($8)
	SB	$5,30
	LDW	$8,&H0092	;LCD2
	JR	YSET
LCD1:	LDW	$8,&H0C82	;LCD1
;
;	set Y address
;
YSET:	LD	$6,$5
	BID	$6
	JR	C,YST01
	PRE	IZ,WHITE	;set white key
	LD	$10,3,XSET	;Y=$10 (line 4)
YST01:	LD	$10,2		;Y=$10 (line 3)
	AD	$9,6
;
;	set X address
;
XSET:	BIU	$6		;
	BIU	$6		;
	LD	$7,$6		;X = $9 + $6 * 12
	BIU	$7		;
	AD	$7,$6		;
	AD	$9,$7		;
	SBC	$9,96		;check LCD block
	JR	C,DRW01		;
	AD	$9,32		;add offset for block2
DRW01:	CAL	LCDST		;drawing
	STLM	$15,5		;
	RTN
;
;	erase finger
;
ERASE:	PST	UA,&H55
	LDM	$15,(IZ+$31),5	;load key graphic data
	SBC	$5,29		;check G#2 key
	JR	NZ,DRW01	;
GS2DR:	LD	$10,2		;drawing for G#2 key
	LDW	$8,&HDA82	;LCD1
	CAL	LCDST
	STLM	$15,3
	LDW	$8,&H0092	;LCD2
	CAL	LCDST
	STLW	$18
	RTN

#if MODEL=FX870P
;
;	set indicator
;
CAPST:	LD	$10,$31,SET01	;CAPS
BASST:	LD	$10,$30,SET01	;BASIC 
ANGST:  LD	$10,2		;DEG/RAD
SET01:	LDW	$8,&HDE92
#endif
;
;	set LCD command
;
LCDST:	PPO	&HC5
	STLM	$8,3
	PPO	&HC4
	RTN
;
;	set beep
;
BPSET:	PRE	IX,TNDAT	;
	LD	$5,$0		;load tone data
	BIU	$0		;
	LDW	$6,(IX+$0)	;
	GST	PD,$2		;
	AN	$2,&H3F		;set PD resitor = 01xxxxxxB
	OR	$2,&H40		;
BPS01:	PST	UA,&H45		;IX = bank0
	PRE	IY,&H0300	;set bottom address of search area
	RTN
;
;	check [shift] key
;
SFTCK:	GST	IA,$2
	PST	IA,&H2
	NOP
	GRE	KY,$0
	ANC	$0,&H04
	PST	IA,$2
	RTN
;
;	key matrix data
;
KYDAT:	DB	&H8,&H80,&H7,&H80,&H9,&H80,&H6,&H40	; CAPS    A      Z      S
	DB	&H8,&H40,&H1,&H01,&H9,&H40,&H6,&H20	;  X             C      F
	DB	&H8,&H20,&H7,&H20,&H9,&H20,&H1,&H80	;  V      G      B     BRK
	DB	&H8,&H10,&H7,&H10,&H9,&H10,&H6,&H08	;  N      J      M      K
	DB	&H8,&H08,&H7,&H08,&H9,&H08,&H1,&H01	;  ,      L      =
	DB	&H8,&H04,&H6,&H04,&H9,&H04,&H7,&H04	; SPC    ANS     0      1
	DB	&H9,&H02,&H1,&H01,&H9,&H01,&H8,&H01	;  .             |E     3
	DB	&H4,&H80,&H3,&H80,&H5,&H80,&H2,&H40	;  Q     MEMO    W      IN
	DB	&H4,&H40,&H1,&H01,&H5,&H40,&H2,&H20	;  E             R     CALC
	DB	&H4,&H20,&H3,&H20,&H5,&H20,&H1,&H01	;  T           Y
	DB	&H4,&H10,&H3,&H10,&H5,&H10,&H2,&H08	;  U           I      
	DB	&H4,&H08,&H3,&H08,&H5,&H08,&H1,&H01	;  O     INS     P
	DB	&H4,&H04,&H2,&H04,&H5,&H04,&H3,&H04	; ENG   SHIFT    4      7
	DB	&H7,&H02,&H1,&H01,&H7,&H01,&H6,&H01	;  5             6      9
	DB	&H4,&H02,&H3,&H02,&H5,&H02,&H3,&H01	;  MR    log     M+     ln

	DB	&H7,&H80,&H8,&H80,&H9,&H80,&H8,&H40	;  BS     *      +      -

KYDT2:	DB	&H2,&H08,&H3,&H10,&H3,&H20,&H2,&H10	;                 
	DB	&H1,&H80,&H2,&H20,&H7,&H40		;  BRK   CALC    /
;
;	tone data
;
TNDAT:
#if MODEL=FX870P
	DW	&H06D,&H092,&H0B6,&H0D7,&H0F7,&H000	; G0  G#0  A0  A#0  B0  --
	DW	&H115,&H131,&H14C,&H165,&H17D,&H000	; C1  C#1  D1  D#1  E1  --
	DW	&H193,&H1A8,&H1BC,&H1CF,&H1E1,&H1F2	; F1  F#1  G1  G#1  A1  A#1
	DW	&H201,&H000,&H210,&H21F,&H22C,&H238	; B1  --   C2  C#2  D2  D#2
	DW	&H244,&H000,&H24F,&H25A,&H264,&H26D	; E2  --   F2  F#2  G2  G#2
	DW	&H276,&H27F,&H287,&H000,&H28E,&H295	; A2  A#2  B2  --   C3  C#3
	DW	&H29C,&H2A2,&H2A8,&H000,&H2AE,&H2B3	; D3  D#3  E3  --   F3  F#3
	DW	&H2B8,&H2BD,&H2C1,&H2C5,&H2C9,&H000	; G3  G#3  A3  A#3  B3  --
	DW	&H2CD,&H2D1,&H2D4,&H2D7,&H2DA,&H000	; C4  C#4  D4  D#4  E4  --
	DW	&H2DD,&H2DF,&H2E2,&H2E4,&H2E6,&H2E9	; F4  F#4  G4  G#4  A4  A#4
#else
	DW	&H075,&H09A,&H0BD,&H0DE,&H0FD,&H000	; G0  G#0  A0  A#0  B0  --
	DW	&H11B,&H137,&H151,&H16A,&H182,&H000	; C1  C#1  D1  D#1  E1  --
	DW	&H198,&H1AD,&H1C0,&H1D3,&H1E4,&H1F5	; F1  F#1  G1  G#1  A1  A#1
	DW	&H205,&H000,&H213,&H221,&H22F,&H23B	; B1  --   C2  C#2  D2  D#2
	DW	&H247,&H000,&H252,&H25C,&H266,&H26F	; E2  --   F2  F#2  G2  G#2
	DW	&H278,&H280,&H288,&H000,&H290,&H297	; A2  A#2  B2  --   C3  C#3
	DW	&H29D,&H2A3,&H2A9,&H000,&H2AF,&H2B4	; D3  D#3  E3  --   F3  F#3
	DW	&H2B9,&H2BE,&H2C2,&H2C6,&H2CA,&H000	; G3  G#3  A3  A#3  B3  --
	DW	&H2CE,&H2D1,&H2D5,&H2D8,&H2DB,&H000	; C4  C#4  D4  D#4  E4  --
	DW	&H2DD,&H2E0,&H2E2,&H2E5,&H2E7,&H2E9	; F4  F#4  G4  G#4  A4  A#4
#endif
;
;	length data
;
LGDAT:	DW	049,052,055,058,062,000			; G0  G#0  A0  A#0  B0  --
	DW	065,069,073,078,083,000			; C1  C#1  D1  D#1  E1  --
	DW	087,093,098,104,110,117			; F1  F#1  G1  G#1  A1  A#1
	DW	124,000,131,139,147,156			; B1  --   C2  C#2  D2  D#2
	DW	165,000,175,185,196,207			; E2  --   F2  F#2  G2  G#2
	DW	220,233,247,000,263,278			; A2  A#2  B2  --   C3  C#3
	DW	293,310,329,000,350,370			; D3  D#3  E3  --   F3  F#3
	DW	393,418,440,466,494,000			; G3  G#3  A3  A#3  B3  --
	DW	526,553,593,627,666,000			; C4  C#4  D4  D#4  E4  --
	DW	694,742,777,837,883,933			; F4  F#4  G4  G#4  A4  A#4
;
;	initial graphic order data
;
ORDER:
#if MODEL=FX870P
	DB	&H06,&H06,&H06,&H0C,&H00,&H06,&H0C,&H00 ;line 2
	DB	&H06,&H06,&H0C,&H00,&H06,&H0C,&H00,&H06
	DB	&H06,&H0C,&H00,&H06,&H0C,&H00,&H06,&H06
	DB	&H0C,&H00,&H06,&H0C,&H00,&H06,&H06,&H06

	DB	&H24,&H18,&H18,&H1E,&H12,&H18,&H1E,&H12 ;line 3
	DB	&H18,&H18,&H1E,&H12,&H18,&H1E,&H12,&H18
	DB	&H18,&H1E,&H12,&H18,&H1E,&H12,&H18,&H18
	DB	&H1E,&H12,&H18,&H1E,&H12,&H18,&H18,&H36

	DB	&H2A,&H30,&H30,&H30,&H30,&H30,&H30,&H30 ;line 4
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H3C
#else
	DB	&H06,&H06,&H06,&H0C,&H00,&H06,&H0C,&H00 ;line 2
	DB	&H06,&H06,&H0C,&H00,&H06,&H0C,&H00,&H06
	DB	&H06,&H0C,&H00,&H06,&H0C,&H00,&H06,&H06
	DB	&H0C,&H00,&H06,&H0C,&H00,&H06,&H06,&H0C

	DB	&H24,&H18,&H18,&H1E,&H12,&H18,&H1E,&H12 ;line 3
	DB	&H18,&H18,&H1E,&H12,&H18,&H1E,&H12,&H18
	DB	&H18,&H1E,&H12,&H18,&H1E,&H12,&H18,&H18
	DB	&H1E,&H12,&H18,&H1E,&H12,&H18,&H18,&H1E

	DB	&H2A,&H30,&H30,&H30,&H30,&H30,&H30,&H30 ;line 4
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
	DB	&H30,&H30,&H30,&H30,&H30,&H30,&H30,&H30
#endif
;
;	graphic data
;
CGDAT:	DB	&H00,&H00,&H00,&H00,&HFF,&HFF		; 1 (&H00)
	DB	&HFF,&H00,&H00,&H00,&HFF,&HFF		; 2 (&H06)
	DB	&HFF,&H00,&H00,&H00,&H00,&HFF		; 3 (&H0C)
	DB	&H00,&H00,&H00				; 4 (&H12)
BLACK:	DB	&H00,&HFC,&HFF				; 4
	DB	&HFC,&H00,&H00,&H00,&HFC,&HFF		; 5 (&H18)
	DB	&HFC,&H00,&H00,&H00,&H00,&HFF		; 6 (&H1E)
	DB	&HFF,&H00,&H00,&H00,&HFC,&HFF		; 7 (&H24)
	DB	&HFE,&H01,&H01,&H01,&H01,&HFE		; 8 (&H2A)
WHITE:	DB	&H01,&H01,&H01,&H01,&H01,&HFE		; 9 (&H30)

#if MODEL=FX870P
	DB	&HFC,&H00,&H00,&H00,&HFF,&H00		;10 (&H36)
	DB	&H01,&H01,&H01,&H01,&HFE,&H00		;11 (&H3C)
#endif
;
;	file name of music data
;
FNAME:	DB	'SF"MMC.BIN"',0				;file search command
