;************************************************************
;*  Electric keyboard emulator for FX-870P/VX-4/VX-3 Ver.1  *
;*          (C)2009 Miyura (miyura321@ybb.ne.jp)            *
;************************************************************

FX870P:	EQU	1		;FX-870P
VX3:	EQU	2		;VX-3

MODEL:	EQU	FX870P

#if MODEL=FX870P
BASE:	EQU	&H123C		;LCD output buffer
#else
BASE:	EQU	&H6343		;LCD output buffer
#endif

	ORG	BASE
	START	INIT
;
;	draw keyboard graphic
;
INIT:	PRE	IX,CGDAT
	PRE	IZ,ORDER
	XRW	$9,$9
	LD	$8,&H82		;draw keyboard graphic (left side)
	CAL	DRWKB		;
	LD	$8,&H92		;draw keyboard graphic (right side)
	CAL	DRWKB		;
	PRE	IY,&H0300	;set bottom address of search area
;
;	scan input key
;
KYSCN:	PRE	IX,KYDAT
	NA	$0,$31		;$0 <- FFh
KYS01:	LDIW	$11,(IX+$31)	;$11=IA, $12=KY
	PST	IA,$11
	AD	$0,$30		;$0=$0+1
	SBC	$0,60
	JR	Z,KYSCN
	GRE	KY,$3
	SBC	$3,$12		;check keycode
	JR	NZ,KYS01

	SBC	$0,11		;check [BRK] key
	JR	Z,QUIT

	CAL	DRAW		;draw finger

	PRE	IX,TNDAT	;
	LD	$5,$0		;load tone data
	BIU	$0		;
	LDW	$6,(IX+$0)	;
	GST	PD,$2		;
	AN	$2,&H3F		;set PD resistor = 01xxxxxxB
	OR	$2,&H40		;
	PST	UA,&H45		;IX = bank0
BEP01:	PST	PD,$2		;make a sound
	XR	$2,&HC0		;
	PRE	IX,$6
	SUP	$30		;wait
	GRE	KY,$0		;
	ANC	$3,$0		;key hold check
	JR	NZ,BEP01	;

	PST	UA,&H55		;IX = bank1
	CAL	ERASE		;erase finger
	JR	KYSCN
;
;	termination process
;
QUIT:	PST	UA,&H54
	RTN
;
;	draw finger
;
DRAW:	LDW	$15,&HB07F	;
	LDW	$17,&HB0B0	;load finger graphic data
	LD	$19,&H7F	;
	PRE	IZ,BLACK	;set black key
	LD	$5,$0
	SBC	$0,29		;check G#2
	JR	Z,GS2DR		;
	JR	C,LCD1		;select LCD1/LCD2 ($8)
	SB	$5,30
	LDW	$8,&H0092	;LCD2
	JR	YSET
LCD1:	LDW	$8,&H0C82	;LCD1
;
;	set Y address
;
YSET:	LD	$6,$5
	BID	$6
	JR	C,YST01
	PRE	IZ,WHITE	;set white key
	LD	$10,3,XSET	;Y = $10 (line 4)
YST01:	LD	$10,2		;Y = $10 (line 3)
	AD	$9,6
;
;	set X address
;
XSET:	BIU	$6		;
	BIU	$6		;
	LD	$7,$6		;X = $9 + $6 * 12
	BIU	$7		;
	AD	$7,$6		;
	AD	$9,$7		;
	SBC	$9,96		;check LCD block
	JR	C,DRW01		;
	AD	$9,32		;add offset for block2
DRW01:	CAL	LCDST		;drawing
	STLM	$15,5		;
	RTN
;
;	erase finger
;
ERASE:	LDM	$15,(IZ+$31),5	;load key graphic data
	SBC	$5,29		;check G#2 key
	JR	NZ,DRW01	;
GS2DR:	LD	$10,2		;drawing for G#2 key
	LDW	$8,&HDA82	;LCD1
	CAL	LCDST
	STLM	$15,3
	LDW	$8,&H0092	;LCD2
	CAL	LCDST
	STLW	$18
	RTN
;
;	set LCD command
;
LCDST:	PPO	&HC5
	STLM	$8,3
	PPO	&HC4
	RTN
;
;	draw keyboard (half)
;
DRWKB:	CAL	LCDST
	LD	$1,64
DRKB1:	LDI	$0,(IZ+$31)
	LDM	$2,(IX+$0),6
	STLM	$2,6
	SB	$1,$30
	JR	NZ,DRKB1
	RTN
;
;	key matrix data
;
KYDAT:	DB	&H8,&H80,&H7,&H80,&H9,&H80,&H6,&H40	; CAPS    A      Z      S
	DB	&H8,&H40,&H1,&H01,&H9,&H40,&H6,&H20	;  X             C      F
	DB	&H8,&H20,&H7,&H20,&H9,&H20,&H1,&H80	;  V      G      B     BRK
	DB	&H8,&H10,&H7,&H10,&H9,&H10,&H6,&H08	;  N      J      M      K
	DB	&H8,&H08,&H7,&H08,&H9,&H08,&H1,&H01	;  ,      L      =
	DB	&H8,&H04,&H6,&H04,&H9,&H04,&H7,&H04	; SPC    ANS     0      1
	DB	&H9,&H02,&H1,&H01,&H9,&H01,&H8,&H01	;  .             |E     3
	DB	&H4,&H80,&H3,&H80,&H5,&H80,&H2,&H40	;  Q     MEMO    W      IN
	DB	&H4,&H40,&H1,&H01,&H5,&H40,&H2,&H20	;  E             R     CALC
	DB	&H4,&H20,&H3,&H20,&H5,&H20,&H1,&H01	;  T           Y
	DB	&H4,&H10,&H3,&H10,&H5,&H10,&H2,&H08	;  U           I      
	DB	&H4,&H08,&H3,&H08,&H5,&H08,&H1,&H01	;  O     INS     P
	DB	&H4,&H04,&H2,&H04,&H5,&H04,&H3,&H04	; ENG   SHIFT    4      7
	DB	&H7,&H02,&H1,&H01,&H7,&H01,&H6,&H01	;  5             6      9
	DB	&H4,&H02,&H3,&H02,&H5,&H02,&H3,&H01	;  MR    log     M+     ln
;
;	tone data
;
TNDAT:
#if MODEL=FX870P
	DW	&H06D,&H092,&H0B6,&H0D7,&H0F7,&H000	; G0  G#0  A0  A#0  B0  --
	DW	&H115,&H131,&H14C,&H165,&H17D,&H000	; C1  C#1  D1  D#1  E1  --
	DW	&H193,&H1A8,&H1BC,&H1CF,&H1E1,&H1F2	; F1  F#1  G1  G#1  A1  A#1
	DW	&H201,&H000,&H210,&H21F,&H22C,&H238	; B1  --   C2  C#2  D2  D#2
	DW	&H244,&H000,&H24F,&H25A,&H264,&H26D	; E2  --   F2  F#2  G2  G#2
	DW	&H276,&H27F,&H287,&H000,&H28E,&H295	; A2  A#2  B2  --   C3  C#3
	DW	&H29C,&H2A2,&H2A8,&H000,&H2AE,&H2B3	; D3  D#3  E3  --   F3  F#3
	DW	&H2B8,&H2BD,&H2C1,&H2C5,&H2C9,&H000	; G3  G#3  A3  A#3  B3  --
	DW	&H2CD,&H2D1,&H2D4,&H2D7,&H2DA,&H000	; C4  C#4  D4  D#4  E4  --
	DW	&H2DD,&H2DF,&H2E2,&H2E4,&H2E6,&H2E9	; F4  F#4  G4  G#4  A4  A#4
#else
	DW	&H075,&H09A,&H0BD,&H0DE,&H0FD,&H000	; G0  G#0  A0  A#0  B0  --
	DW	&H11B,&H137,&H151,&H16A,&H182,&H000	; C1  C#1  D1  D#1  E1  --
	DW	&H198,&H1AD,&H1C0,&H1D3,&H1E4,&H1F5	; F1  F#1  G1  G#1  A1  A#1
	DW	&H205,&H000,&H213,&H221,&H22F,&H23B	; B1  --   C2  C#2  D2  D#2
	DW	&H247,&H000,&H252,&H25C,&H266,&H26F	; E2  --   F2  F#2  G2  G#2
	DW	&H278,&H280,&H288,&H000,&H290,&H297	; A2  A#2  B2  --   C3  C#3
	DW	&H29D,&H2A3,&H2A9,&H000,&H2AF,&H2B4	; D3  D#3  E3  --   F3  F#3
	DW	&H2B9,&H2BE,&H2C2,&H2C6,&H2CA,&H000	; G3  G#3  A3  A#3  B3  --
	DW	&H2CE,&H2D1,&H2D5,&H2D8,&H2DB,&H000	; C4  C#4  D4  D#4  E4  --
	DW	&H2DD,&H2E0,&H2E2,&H2E5,&H2E7,&H2E9	; F4  F#4  G4  G#4  A4  A#4
#endif
;
;	initial graphic order data
;
ORDER:	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00 ; LCD1 line 1
	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00
	DB	&H0C,&H0C,&H0C,&H12,&H06,&H0C,&H12,&H06 ; LCD1 line 2
	DB	&H0C,&H0C,&H12,&H06,&H0C,&H12,&H06,&H0C
	DB	&H2A,&H1E,&H1E,&H24,&H18,&H1E,&H24,&H18 ; LCD1 line 3
	DB	&H1E,&H1E,&H24,&H18,&H1E,&H24,&H18,&H1E
	DB	&H30,&H36,&H36,&H36,&H36,&H36,&H36,&H36 ; LCD1 line 4
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36

	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00 ; LCD2 line 1
	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00
	DB	&H0C,&H12,&H06,&H0C,&H12,&H06,&H0C,&H0C	; LCD2 line 2
#if MODEL=FX870P
	DB	&H12,&H06,&H0C,&H12,&H06,&H0C,&H0C,&H0C
	DB	&H1E,&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E ; LCD2 line 3
	DB	&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E,&H3C
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36	; LCD2 line 4
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H42
#else
	DB	&H12,&H06,&H0C,&H12,&H06,&H0C,&H0C,&H12
	DB	&H1E,&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E ; LCD2 line 3
	DB	&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E,&H24
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36	; LCD2 line 4
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36
#endif
;
;	graphic data
;
CGDAT:	DB	&HFF,&HFF,&HFF,&HFF,&HFF,&HFF		; 0 (&H00)
	DB	&H00,&H00,&H00,&H00,&HFF,&HFF		; 1 (&H06)
	DB	&HFF,&H00,&H00,&H00,&HFF,&HFF		; 2 (&H0C)
	DB	&HFF,&H00,&H00,&H00,&H00,&HFF		; 3 (&H12)
	DB	&H00,&H00,&H00				; 4 (&H18)
BLACK:	DB	&H00,&HFC,&HFF				; 4
	DB	&HFC,&H00,&H00,&H00,&HFC,&HFF		; 5 (&H1E)
	DB	&HFC,&H00,&H00,&H00,&H00,&HFF		; 6 (&H24)
	DB	&HFF,&H00,&H00,&H00,&HFC,&HFF		; 7 (&H2A)
	DB	&HFE,&H01,&H01,&H01,&H01,&HFE		; 8 (&H30)
WHITE:	DB	&H01,&H01,&H01,&H01,&H01,&HFE		; 9 (&H36)
#if MODEL=FX870P
	DB	&HFC,&H00,&H00,&H00,&HFF,&H00		;10 (&H3C)
	DB	&H01,&H01,&H01,&H01,&HFE,&H00		;11 (&H42)
#endif